function dataout = PCIcalc2(datain, envOuter, envInner, p)
% PCICALC2 calculates PI: (phyto) plankton (community) index
%
% The transformed abundances of two lifeforms are, notionally
%   plotted against each other, 
%   giving points in a 2-D, Cartesian, lifeform space.
% Outer and inner reference envelopes (defining a 2-D doughnut)
%   are superimposed on this plot.
% PI = number of points outside envelopes, divided by total points
% Significance calculated by exact bionomial and chi-square tests,
%   using a supplied expectation (should be that used to make envelope)
%
% input
%   datain : array, 4 columns, in which
%       col 1 = data, year.dec - not used
%       col 2 = Y1, lifeform 1 transformed abundance = X co-ordinate
%       col 3 = Y2, lifeform 2 transformed abundance = Y co-ordinate
%       col 4 = month - is not used
%   envOuter : array, 2 columns, giving X and Y co-ordinates of
%           outer reference envelope
%   envInner : array, 2 columns, giving X and Y co-ordinates of
%           inner reference envelope 
%           (surrounding the hole in the doughnut)
%   p = scalar, proportion of points that were included in the envelope
%           (excluded proportion is (1-p)/2 beyond outer enevlope and
%           (the same within inner envelope)
% output
%   dataout : vector, in which
%       (1) = PI value
%       (2) = total number of points (data rows)
%       (3) = number of points outside envelope
%       (4) = probability (exact binomial test)
%       (5) = value of chi-quare (1 df)

%
% history and authorship:
%   made by PT, 10 Aug 08, drawing on script PCI1ED2
%   as function dataout = PCIcalc(datain, envOuter, envInner)
%   which used a default expectation, typically 0.95
%   PCIcalc2 made by PT, 9 August 2009, to use a supplied expectation
% ======================================================

if p > 0.5 && p < 1.0, expectation = p; else expectation = 0.95; end
%
% Identify points outside the envelope using the 'inpolygon' function;
%   points inside or on inner envelope get '1' in column 1, those on
%   envelope have '1' in column 2, and so those inside only get '1' 
%   in column 3
[points_new_in(:,1) points_new_in(:,2)]= ...
        inpolygon(datain(:,2), datain(:,3), ...
            envInner(:,1), envInner(:,2));
points_new_in(:,3)=points_new_in(:,1) &~points_new_in(:,2);
% points outside outer envelope get '0', 
%   those on boundary or inside have '1'
points_new_out=inpolygon(datain(:,2), datain(:,3), ...
            envOuter(:,1), envOuter(:,2));
% count external and internal points 
N_outside_new=length(find(points_new_in(:,3)>0.5)) ...
        +length(find(points_new_out<0.5));
N_total_new=length(datain(:,2));
% =========
% the PCI = PI
% =========
pci=1-(N_outside_new/N_total_new);
% binomial probability
warning off;
q_in=expectation;
p_out=1-q_in;
outcomes=zeros(1,N_total_new+1);
for n=0:N_total_new,
	outcomes(n+1)= ...
        nchoosek(N_total_new,n)*p_out^n*q_in^(N_total_new-n);
end;
prob=sum(outcomes(N_outside_new+1:N_total_new+1));
warning on;	
% chi-square
expect=N_total_new*p_out;
chi_square=(N_outside_new-expect)^2/expect;
%
dataout(1) = pci;
dataout(2) = N_total_new;
dataout(3) = N_outside_new;
dataout(4) = prob;
dataout(5) = chi_square;
% end of function